#include "EnumFinder.h"
#include "antlr4-runtime.h"
#include "EtoscLexer.h"
#include "EtoscParser.h"

using namespace antlr4;

unique_ptr<EnumsVector> EnumFinder::find(const string &fileName)
{
    ifstream file(fileName); 
    
    if (!file.is_open())
        throw runtime_error(string("Can't open file ") + fileName);

    ANTLRInputStream input(file);
    EtoscLexer lexer(&input);
    CommonTokenStream tokens(&lexer);
    EtoscParser parser(&tokens);

    EnumFinder enumFinder;
    
    parser.setEnumFinder(&enumFinder);
    parser.setBuildParseTree(false);
    parser.parseFile();

    file.close();

    return unique_ptr<EnumsVector>(enumFinder.enums.release());
}

void EnumFinder::startAddingEnum(const string &enumName)
{
    string namespaces;
    
    for (auto& namespaceName : nestedNamespaces)
        namespaces += namespaceName + "::"; 

    currentEnum = unique_ptr<Enum>(new Enum);
    currentEnum->name = namespaces + enumName;
}

void EnumFinder::endAddingEnum()
{
    enums->push_back(unique_ptr<Enum>(currentEnum.release()));
}

void EnumFinder::addEnumerator(const string &enumeratorName)
{
    currentEnum->enumerators.push_back(enumeratorName);
}
