#include "FilesGenerator.h"

FilesGenerator::FilesGenerator(unique_ptr<EnumsVector> &e, const string &inputFileName) :
    enums(e.release()),
    inputFileName(inFileName) {}


void FilesGenerator::generate()
{
   generateHeaderFile();
   generateSourceFile();
}

void FilesGenerator::generateHeaderFile()
{
    ofstream file(headerFileName);
    
    if (!file.is_open())
        throw runtime_error(string("Can't open file ") + headerFileName);

    file << "#ifndef ETOSC\n"
            "#define ETOSC\n"
            "\n"
            "#include <string>\n"
            "\n"
            "template <typename EnumType>\n"
            "std::string enumToString(EnumType val);\n"
            "\n"
            "template <typename EnumType>\n"
            "EnumType stringToEnum(const std::string &str);\n"
            "\n"
            "#endif\n";
    
    file.close();
}

void FilesGenerator::generateSourceFile()
{
    ofstream file(sourceFileName);
    
    if (!file.is_open())
        throw runtime_error(string("Can't open file ") + sourceFileName);

    file << "#include <stdexcept>\n"
            "\n"
            "#include \"" << headerFileName << "\"\n"
            "#include \"" << inputFileName << "\"\n";

    for (auto& e : *enums)
    {
        string &enumName = e->name;
        
        file << "\n"
                "template<>\n"
                "std::string enumToString(" << enumName << " val)\n"
                "{\n"
                "    switch (val)\n"
                "    {";

        for (auto& enumerator : e->enumerators)
        {
            file << "\n"
                    "        case " << enumName << "::" << enumerator << ":\n"
                    "            return \"" << enumerator << "\";\n";
        }

        file << "    }\n"
                "}\n"
                "\n"
                "template<>\n"
                << enumName << " stringToEnum(const std::string &str)\n"
                "{";
        
        for (auto& enumerator : e->enumerators)
        {
            file << "\n"
                    "    if (str == \"" << enumerator << "\")\n"
                    "        return " << enumName << "::" << enumerator << ";\n";
        }

        file << "\n"
                "    throw std::runtime_error(std::string(\"Enum "
                << enumName << " has no \"\n " 
                "                                        \"enumerator named \") + str);\n"
                "}\n";
    }
    
    file.close();
}

